/* $Id: conf.c,v 1.46 2003/01/09 19:46:07 rawsock Exp $	*/
/* Borrowed from GNU Wget *
 * Modified by rawsock    */

/* include local header files {{{ */
#include "pch.h"
#include "main.h"
#include "match.h"
#include "misc.h"
#include "net.h"
/* }}} */

/* commands recognized by config {{{ */
const char *commands[]={
	"listen_port_main",	// 0
	"listen_port_cons",	// 1
	"hubname",		// 2
	"user_limit",		// 3
	"allow_broken_key",	// 4
	"ping_timeout",		// 5
	"register_timeout",	// 6
	"conf_nallow",		// 7
	"min_share",		// 8
	"log_dir",		// 9
	"log_main",		// 10
	"user",			// 11
	"group",		// 12
	"allow_chat",		// 13
	"conf_welcome",		// 14
	"conf_motd",		// 15
	"conf_cusers",		// 16
	"conf_callow",		// 17
	"conf_banned",		// 18
	""
}; /* }}} */

extern struct config conf;

/* strdupdelim() - copy the string formed by two pointers to a new location {{{ */
char *strdupdelim(const char *beg,const char *end) {
	char *res=(char *)my_malloc(end-beg+1);
	
	memcpy(res,beg,end-beg);
	res[end-beg]=0;
	return res;
} /* }}} */

/* Syntax: <sp>* command <sp>* = <sp>* value <newline>
 * Return values:
 *	1	- success
 *	0	- failure
 *	-1	- empty */ 
/* parse_line() {{{ */
int parse_line(const char *line,char **com,char **val) {
	const char *p=line;
	const char *orig_comptr,*end;
	char *new_comptr;
	
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') return -1;
	for (orig_comptr=p;isalpha(*p)||*p=='_'||*p=='-';p++);
	if (!isspace(*p)&&(*p!='=')) return 0;
	*com=(char *)my_malloc(p-orig_comptr+1);
	for (new_comptr=*com;orig_comptr<p;orig_comptr++)
		*new_comptr++ = *orig_comptr;
	*new_comptr=0;
	for (;isspace(*p);p++);
	if (*p!='=') { 
		my_free (*com); 
		return 0;
	}
	for (++p;isspace(*p);p++);
	end=line+strlen(line)-1;
	while(end>p&&isspace(*end)) --end;
	*val=strdupdelim(p,end+1);
	return 1;
} /* }}} */

/* read_whole_line() - read a line from FP and return the pointer to allocated storage. {{{ */
char *read_whole_line(FILE *fp) {
	int length=0;
	int bufsize=82;
	char *line=(char *)my_malloc(bufsize);
	
	while(fgets(line+length,bufsize-length,fp))	{
		length+=strlen(line+length);
		if (length==0) continue;
		if (line[length-1]=='\n') break;
		bufsize<<=1;
		line=(char *)my_realloc(line,bufsize);
	}
	if (length==0||ferror(fp)) {
		my_free (line);
		return NULL;
	}
	if (length+1<bufsize) line=(char *)my_realloc(line,length+1);
	return line;
} /* }}} */

/* parse_config() - parse config file {{{ */
char parse_config(struct config *conf) {
    char *line,tmp[PATH_MAX];
    int ln;
    FILE *fp;
    struct passwd *user_data;
    struct group *group_data;

    /* opening config {{{ */
    /* we look for config in location taken from configure or from
       command-line option */
    if (!realpath(conf->conf_main,tmp)) {
    	debug(LOG_WARNING,0,"Can't find config file (%s): %s.",conf->conf_main, strerror(errno));
    	return 1;
    }
    my_free(conf->conf_main);
    conf->conf_main=strdup(tmp);
    fp=fopen(conf->conf_main,"rb");
    if (!fp) {
    	debug(LOG_WARNING,0,"Can't open config file (%s): %s.",conf->conf_main, strerror(errno));
    	return 1;
    }
    /* }}} */

    ln=1;
    while((line=read_whole_line(fp))) {
    	char *com,*val;
    	int status;

	status=parse_line(line,&com,&val);
	my_free(line);
	if (status==1) {
	    status=0;
	    while(*commands[status]) { 
		if (!strcasecmp(com,commands[status])) break;
		else status++;
	    }
	    /* Options {{{ */
	    switch(status) {
		/* 0. listen_port_main {{{ */
		case 0: if ((status=atoi(val))==0) {
			    debug(-2,0,"Error in %s at line %d.",conf->conf_main,ln);
			    my_free(com);
			    my_free(val);
			    fclose(fp);
			    return 1;
			}
			conf->listen_port_main=status;
			break; /* }}} */
		/* 1. listen_port_cons {{{ */
		case 1: conf->listen_port_cons=atoi(val);
			break; /* }}} */
		/* 2. hubname {{{ */
		case 2: my_free(conf->hubname);
			conf->hubname=strdup(val);
			break; /* }}} */
		/* 3. user_limit {{{ */
		case 3: conf->userlimit=atoi(val);
			break; /* }}} */
		/* 4. allow_broken_key {{{ */
		case 4: conf->allow_broken_key=atoi(val);
			break; /* }}} */
		/* 5. ping_timeout {{{ */
		case 5: conf->ping_timeout=atoi(val);
			break; /* }}} */
		/* 6. register_timeout {{{ */
		case 6: conf->register_timeout=atoi(val);
			break; /* }}} */
		/* 7. allowed nicks file {{{ */
		case 7: my_free(conf->conf_nallow);
		        conf->conf_nallow=strdup(val);
		        break; /* }}} */
		/* 8. min_share {{{ */
		case 8: conf->minshare=atoi(val);
			break; /* }}} */
		/* 9. log files dir {{{ */
		case 9: my_free(conf->log_dir);
			conf->log_dir=strdup(val);
			break; /* }}} */
		/* 10. main log file name {{{ */
		case 10: my_free(conf->log_main);
			 conf->log_main=strdup(val);
			 break; /* }}} */
		/* 11. UID to run {{{ */
		case 11: user_data=getpwnam(val);
		         if(!user_data) {
			    debug(-2,0,"Error in %s at line %d: user `%s' not found.",conf->conf_main,ln,val);
			    my_free(com);
			    my_free(val);
			    fclose(fp);
			    return 1;
		    	 }
		         conf->uid=user_data->pw_uid;
		         break; /* }}} */
		/* 12. GID to run {{{ */
		case 12: group_data=getgrnam(val);
		         if(!group_data) {
			    debug(-2,0,"Error in %s at line %d: group `%s' not found.",conf->conf_main,ln,val);
			    my_free(com);
			    my_free(val);
			    fclose(fp);
			    return 1;
		         }
		         conf->gid=group_data->gr_gid;
		         break; /* }}} */
		/* 13. allow chat {{{ */
		case 13: conf->allow_chat=atoi(val);
		         break; /* }}} */
		/* 14. welcome message file {{{ */
		case 14: my_free(conf->conf_welcome);
		    	 conf->conf_welcome=strdup(val);
			 break; /* }}} */
		/* 15. message of the day {{{ */
		case 15: my_free(conf->conf_motd);
		         conf->conf_motd=strdup(val);
		         break; /* }}} */
		/* 16. console.users {{{ */
		case 16: my_free(conf->conf_cusers);
		         conf->conf_cusers=strdup(val);
		         break; /* }}} */
		/* 17. console.allow {{{ */
		case 17: my_free(conf->conf_callow);
		         conf->conf_callow=strdup(val);
		         break; /* }}} */
		/* 18. bans file {{{ */
		case 18: my_free(conf->conf_banned);
		         conf->conf_banned=strdup(val);
		         break; /* }}} */
		/* action when everything fails: {{{ */
		default: debug(-2,0,"Unknown keyword in %s at line %d: %s.",conf->conf_main,ln,com);
			 my_free(com);
		         my_free(val);
		         fclose(fp);
			 return 1; /* }}} */
	    } /* }}} */
	    my_free(com);
	    my_free(val);
	}
	else if (status==0) {
		debug(-2,0,"Error in %s at line %d.",conf->conf_main,ln);
		fclose(fp);
		return 1;
	     }
	++ln;
    }
    fclose(fp);
    return 0;
} /* }}} */

/* playfile() - parse text file and play to public chat window {{{ */
void playfile(struct userrec *usr,char *msgfile) {
    char *line,*p;
    FILE *fp;

    seteuid(0);
    setegid(0);
    fp=fopen(msgfile,"rb");
    seteuid(conf.uid);
    setegid(conf.gid);
    if (!fp) {
        debug(-1,0,"Can't open message file (%s): %s.",msgfile,strerror(errno));
        return;
    }

    while((line=read_whole_line(fp))) {
	p=line;
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') { my_free(line); continue; }
	strtok(p,"\r\n");
	publicmsg(usr,p);
        my_free(line);
    }

    fclose(fp);
} /* }}} */

/* validlogin() - verify login and password {{{ */
char validlogin(struct userrec *usr,char *login,char *pass) {
    const char *sep=":\r\n";
    char *line,*p;
    FILE *fp;

    seteuid(0);
    setegid(0);
    fp=fopen(conf.conf_cusers,"rb");
    seteuid(conf.uid);
    setegid(conf.gid);
    if (!fp) {
        debug(-1,0,"Can't open console users file (%s): %s.",conf.conf_cusers,strerror(errno));
        return 0;
    }

    while((line=read_whole_line(fp))) {
	p=line;
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') { my_free(line); continue; }
	p=strtok(p,sep);
	if (!p) goto leave;
	if (!strcmp(p,login)) {
	    if (usr->cons==0) return 1;
	    p=strtok(NULL,sep); if (!p) goto leave;
	    if (!strcmp(p,pass)) {
		p=strtok(NULL,sep); if (!p) goto leave;
	        my_free(usr->email);
	        usr->email=strdup(p);
	        p=strtok(NULL,sep); if (!p) goto leave;
	        my_free(usr->desc);
	        usr->desc=strdup(p);
	        p=strtok(NULL,sep); if (!p) goto leave;
	        my_free(usr->perm);
	        usr->perm=strdup(p);
    		my_free(line);
	        fclose(fp);
	        return 1;
	    }
	}
    }

    leave:
        my_free(line);
	fclose(fp);
        return 0;
} /* }}} */

/* validhost() - verify host access {{{ */
char validhost(struct userrec *usr,char *access_file,char *ip) {
    const char *sep=":\r\n";
    char *line,*p;
    FILE *fp;

    seteuid(0);
    setegid(0);
    fp=fopen(access_file,"rb");
    seteuid(conf.uid);
    setegid(conf.gid);
    if (!fp) {
        debug(-1,0,"Can't open allow/deny file (%s): %s.",access_file,strerror(errno));
        return 0;
    }

    while((line=read_whole_line(fp))) {
	p=line;
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') { my_free(line); continue; }
	p=strtok(p,sep);
	if (!p) goto leave;
	if (match(p,ip)) {
	    if (usr->cons==0) {
		p=strtok(NULL,sep);
		if (p) publicmsg(usr,p);
		else publicmsg(usr,"Your host is banned.");
	    }
	    my_free(line);
	    fclose(fp);
	    return 1;
	}
    }

    if (usr->cons) publicmsg(usr,"You're not allowed to use the console.");
    leave:
        my_free(line);
	fclose(fp);
        return 0;
} /* }}} */

/* nick_allowed() - verify nick access {{{ */
char nick_allowed(struct userrec *usr,char *nick) {
    const char *sep=" \t\r\n";
    char *line,*p;
    FILE *fp;

    seteuid(0);
    setegid(0);
    fp=fopen(conf.conf_nallow,"rb");
    seteuid(conf.uid);
    setegid(conf.gid);
    if (!fp) {
        debug(-1,0,"Can't open nicks allow file (%s): %s.",conf.conf_nallow,strerror(errno));
        return 1; // allow by default
    }

    while((line=read_whole_line(fp))) {
	p=line;
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') { my_free(line); continue; }
	p=strtok(p,sep);
	if (!p) goto leave;
	if (match(p,nick)) {
	    my_free(line);
	    fclose(fp);
	    return 1;
	}
    }

    leave:
        my_free(line);
	fclose(fp);
	publicmsg(usr,"Your nick doesn't match any of the allowed patterns.");
        return 0;
} /* }}} */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

